#define packages to install
rpackages <- c('tidyverse','sampling','partykit',"tidyr",'magrittr','precrec','ebmc',
              'caret','DMwR','MatchIt','pROC','synthpop','zeallot')
#install all packages that are not already installed
install.packages(setdiff(rpackages, rownames(installed.packages())))
lapply(rpackages, require, character.only = TRUE)

install.packages("tidyverse")
###############################################################
library(tidyverse)
#library(sdgs) # replaced this library with synthpop r package if sdgs is not
# publicly accessible
library(synthpop)
library(sampling)
library(partykit)
library(tableone)
library(tidyr)
library(ggpubr)
library(magrittr)
library(sjPlot)
library(sdgd)
library(precrec)
library(ebmc) # ensemble based learning
library(caret)
library(DMwR)
library(MatchIt)
library(pROC)
library(zeallot) 

######### Define the functions ################

## standard logistic regression model
LRstandard<-function(trainsample,testsample){
  cols_real<- colnames(trainsample)
  datt<-trainsample
  n = dim(datt)[1]; p = dim(datt)[2]
  
  colnames(datt)<-c("Y",paste0("X",1:(p-1)))
  colnames(testsample)<-c("Y",paste0("X",1:(p-1)))
  
  # format the columns
  cols<-colnames(datt)
  col_cat<-colnames(datt)[1:2]
  datt %<>% mutate_at(col_cat, factor)
  
  modelout <- model <- glm(Y~., data = datt, family = binomial)
  modelcoef<-summary(model)$coef
  
  # Make predictions
  datt<-testsample
  datt$scores <- model %>% predict(testsample, type = "response")
  
  # Calculate ROC and Precision-Recall curves
  sscurves <- evalmod(scores = datt$scores, labels = datt$Y,)
  roc1 <- ci.auc(datt$Y,datt$scores)
  auc = attr(sscurves$rocs[[1]],"auc")
  measures <- evalmod(mode="basic", scores = datt$scores, labels = datt$Y)
  sp = median(na.omit(measures$sp[[1]]$y))
  sn = median(na.omit(measures$sn[[1]]$y))
  fscore = median(na.omit(measures$fscore[[1]]$y))
  
  # Show a Precision-Recall plot
  allscore = data.frame(
    matrix(round(c(auc,sp,sn,fscore),4),nrow = 1),
    row.names = NULL)
  colnames(allscore)<-c("auc","sp","sn","fscore")
  allscore<-apply(allscore,2,as.numeric)
  
  return(list(allscore,modelcoef,sscurves,cols_real,modelout,roc1))
}

# random oversampling of the minority group
ros <- function(data) {
  if (mean(data$y) > 0.5) { # check the majority class
    maxClass <- 1
  } else {
    maxClass <- 0
  }
  
  top <- sum(data$y == maxClass)   # size of the majority group
  min <- data[data$y != maxClass,] # select minority group data
  ind <- seq(from = 1, to = nrow(min), by = 1) # create vector with row indices minority group to sample from
  x <- sample(ind, size = top - nrow(min), replace = TRUE) # sample from row indices with replacement 
  #with the size being the difference between the minority and majority class
  data <- rbind(data, min[x,]) # combine old data with duplicated of the sampled minority cases
  data # return over sampled data set
}

# random undersampling of the minority group
rus <- function(data) {
  if (mean(data$y) < 0.5) { # determine which class is the minority class
    minClass <- 1
  } else {
    minClass <- 0
  }
  top <- sum(data$y == minClass) # Determine size minority class
  maj <- data[data$y != minClass,] # Select majority group
  ind <- seq(from = 1, to = nrow(maj), by = 1) # Create vector with indices for the majority group
  x <- sample(ind, size = nrow(maj) - top, replace = FALSE) # sample indices that need to be removed from the original data set
  data <- rbind(data[data$y == minClass,], maj[-x,]) # bind minority class data with remaining majority class data
  data # return undersampled data
}


# RUS for our simulated data setting (takes order of variables into account)
RUS_bias<-function(traindata){
  cols<-colnames(traindata)
  trt_level<-levels(traindata[,2])
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("y",paste0("x",1:(p-1)))[c(2,1,3:p)] #sort for rus
  levels(data$y)<-c("0","1")
  data$y<-ifelse(data$y%in%"0",0,1)
  rdata<-rus(data)            # apply rus on the treatment group
  rdata$y<-as.factor(rdata$y) # relevel the treatment to original
  levels(rdata$y)<-trt_level
  colnames(rdata)<-cols
  return(rdata)
}


# ROS defined for the simulated data 
ROS_bias<-function(traindata){
  cols<-colnames(traindata)
  trt_level<-levels(traindata[,2])
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("y",paste0("x",1:(p-1)))[c(2,1,3:p)] #sort for rus
  levels(data$y)<-c("0","1")
  data$y<-ifelse(data$y%in%"0",0,1)
  rdata<-ros(data)            # apply ros on the treatment group
  rdata$y<-as.factor(rdata$y) # relevel the treatment to original
  levels(rdata$y)<-trt_level
  colnames(rdata)<-cols
  return(rdata)
}

# SMOTE for the biased simulated data
SMOTE_bias<-function(traindata,perc=100){
  cols<-colnames(traindata)
  trt_level<-levels(traindata[,2])
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("y",paste0("x",1:(p-1)))[c(2,1,3:p)] #sort for rus
  levels(data$y)<-c("0","1")
  data$y<-as.factor(ifelse(data$y%in%"0",0,1))
  rdata<-SMOTE(y ~ ., data, perc.over = perc)  # apply smote from DMwR
  rdata$y<-as.factor(rdata$y)                    
  levels(rdata$y)<-trt_level
  colnames(rdata)<-cols
  return(rdata)
}

# Propensity score matching for our biased data (specific to the minority group)
LR_PSbias<-function(trainsample,testsample){
  cols_real<- colnames(trainsample)
  datt<-trainsample
  n = dim(datt)[1]; p = dim(datt)[2]
  
  colnames(datt)<-c("Y",paste0("X",1:(p-1)))
  # format the columns
  colnames(testsample)<-cols<-colnames(datt)
  col_cat<-colnames(datt)[1:2]
  datt %<>% mutate_at(col_cat, factor)
  
  # repeat for the testset
  testsample %<>% mutate_at(col_cat, factor)
  
  Trt<-"X1"
  Variables<-cols[3:p]
  f<-as.formula(
    paste(Trt,paste(Variables,collapse = "+"),sep = "~")
  )
  m.out <- matchit(f, data = datt,
                   method = "nearest", distance = "glm")
  datt <- match.data(m.out)
  # repeat for testset
  t.out <- matchit(f, data = testsample,
                   method = "nearest", distance = "glm")
  testsample <- match.data(t.out)
  ###
  Outcome<-"Y"
  Variables<-cols[2:p]
  fmodel<-as.formula(
    paste(Outcome,paste(Variables,collapse = "+"),sep = "~")
  )
  modelout <- model <- glm(fmodel, data = datt, family = binomial,
                           weights = weights)
  modelcoef<-summary(model)$coef
  
  # Make predictions
  datt<-testsample
  datt$scores <- model %>% predict(testsample, type = "response")
  
  # Calculate ROC and Precision-Recall curves
  sscurves <- evalmod(scores = datt$scores, labels = datt$Y)
  roc1 <- ci.auc(datt$Y,datt$scores)
  
  
  auc = attr(sscurves$rocs[[1]],"auc")
  measures <- evalmod(mode="basic", scores = datt$scores, labels = datt$Y)
  sp = median(na.omit(measures$sp[[1]]$y))
  sn = median(na.omit(measures$sn[[1]]$y))
  fscore = median(na.omit(measures$fscore[[1]]$y))
  
  # Show a Precision-Recall plot
  #autoplot(sscurves, "PRC")
  
  allscore = data.frame(
    matrix(round(c(auc,sp,sn,fscore),4),nrow = 1),
    row.names = NULL)
  colnames(allscore)<-c("auc","sp","sn","fscore")
  allscore<-apply(allscore,2,as.numeric)
  
  return(list(allscore,modelcoef,sscurves,cols_real,modelout,roc1))
}

# Propensity score matching for the majority group
LR_PSbias2<-function(trainsample,testsample,nclass){
  cols_real<- colnames(trainsample)
  datt<-trainsample
  n = dim(datt)[1]; p = dim(datt)[2]
  
  colnames(datt)<-c("Y",paste0("X",1:(p-1)))
  # format the columns
  colnames(testsample)<-cols<-colnames(datt)
  col_cat<-colnames(datt)[1:2]
  datt %<>% mutate_at(col_cat, factor)
  
  # repeat for the testset
  testsample %<>% mutate_at(col_cat, factor)
  
  
  Trt<-"X1"
  Variables<-cols[3:p]
  f<-as.formula(
    paste(Trt,paste(Variables,collapse = "+"),sep = "~")
  )
  m.out <- matchit(f, data = datt,
                   method = "nearest", distance = "glm")
  datt <- match.data(m.out)
  # repeat for testset
  t.out <- matchit(f, data = testsample,
                   method = "nearest", distance = "glm")
  testsample <- match.data(t.out)
  ###
  Outcome<-"Y"
  Variables<-cols[2:p]
  fmodel<-as.formula(
    paste(Outcome,paste(Variables,collapse = "+"),sep = "~")
  )
  modelout <- model <- glm(fmodel, data = datt, family = binomial,
                           weights = weights)
  modelcoef<-summary(model)$coef
  
  # Make predictions
  testsample<-testsample[testsample$X1==levels(testsample$X1)[nclass],]
  datt<-testsample
  datt$scores <- model %>% predict(testsample, type = "response")
  
  # Calculate ROC and Precision-Recall curves
  sscurves <- evalmod(scores = datt$scores, labels = datt$Y)
  roc1 <- ci.auc(datt$Y,datt$scores)
  
  
  auc = attr(sscurves$rocs[[1]],"auc")
  measures <- evalmod(mode="basic", scores = datt$scores, labels = datt$Y)
  sp = median(na.omit(measures$sp[[1]]$y))
  sn = median(na.omit(measures$sn[[1]]$y))
  fscore = median(na.omit(measures$fscore[[1]]$y))
  
  # Show a Precision-Recall plot
  
  allscore = data.frame(
    matrix(round(c(auc,sp,sn,fscore),4),nrow = 1),
    row.names = NULL)
  colnames(allscore)<-c("auc","sp","sn","fscore")
  allscore<-apply(allscore,2,as.numeric)
  
  return(list(allscore,modelcoef,sscurves,cols_real,modelout,roc1))
}


# Modified RF ensembles for the minority group
LR_RFbias<-function(trainsample,testsample){
  cols_real<- colnames(trainsample)
  datt<-trainsample
  n = dim(datt)[1]; p = dim(datt)[2]
  
  colnames(datt)<-c("Y",paste0("X",1:(p-1)))
  # format the columns
  colnames(testsample)<-cols<-colnames(datt)
  col_cat<-colnames(datt)[1:2]
  datt %<>% mutate_at(col_cat, factor)
  
  # repeat for the testset
  testsample %<>% mutate_at(col_cat, factor)
  
  
  model <- adam2(Y ~ ., data = datt, size = 10, alg = "rf",
                 rf.ntree = 200)
  
  datt<-testsample
  prob <- predict(model, newdata = datt, type = "prob")
  # Calculate measurements
  auc <- measure(label = datt$Y, probability = prob,
                 metric = "auc")
  roc1 <- ci.auc(datt$Y,prob)
  tpr <- measure(label = datt$Y, probability = prob,
                 metric = "tpr")
  tnr <- measure(label = datt$Y, probability = prob,
                 metric = "tnr")
  f <- measure(label = datt$Y, probability = prob,
               metric = "f")
  allest<-data.frame(measure=c("auc","tpr","tnr","fscore"),
                     value = round(c(auc,tpr,tnr,f),4))
  return(list(allest,model,roc1))
}

# Modified RF model for the majority group
LR_RFbias2<-function(trainsample,testsample){
  cols_real<- colnames(trainsample)
  datt<-trainsample
  n = dim(datt)[1]; p = dim(datt)[2]
  
  colnames(datt)<-c("Y",paste0("X",1:(p-1)))
  # format the columns
  colnames(testsample)<-cols<-colnames(datt)
  col_cat<-colnames(datt)[1]
  datt %<>% mutate_at(col_cat, factor)
  
  # repeat for the testset
  testsample %<>% mutate_at(col_cat, factor)
  
  
  model <- adam2(Y ~ ., data = datt, size = 10, alg = "rf",
                 rf.ntree = 200)
  
  datt<-testsample
  prob <- predict(model, newdata = datt, type = "prob")
  # Calculate measurements
  auc <- measure(label = datt$Y, probability = prob,
                 metric = "auc")
  roc1 <- ci.auc(datt$Y,prob)
  tpr <- measure(label = datt$Y, probability = prob,
                 metric = "tpr")
  tnr <- measure(label = datt$Y, probability = prob,
                 metric = "tnr")
  f <- measure(label = datt$Y, probability = prob,
               metric = "f")
  allest<-data.frame(measure=c("auc","tpr","tnr","fscore"),
                     value = round(c(auc,tpr,tnr,f),4))
  return(list(allest,model,roc1))
}

# Functions for estimating the 95% Confidence Interval Overlaps 
interval_overlap<-function(L1,U1,L2,U2){
  if (L1==L2 & U1==U2) {
    I = 1
  }
  if (U1<L2 | U2<L1) {
    I = 0
  }
  if (L1<=L2 & L2<U1 & U1<=U2) {
    Li = L2
    Ui = U1
    I = (Ui-Li)/(2*(U1-L1)) + (Ui-Li)/(2*(U2-L2))
  }
  if (L1<=L2 & L2<U1 & U2<U1) {
    Li = L2
    Ui = U2
    I = (Ui-Li)/(2*(U1-L1)) + (Ui-Li)/(2*(U2-L2))
  }
  
  if (L2<=L1 & L1<U2 & U2<=U1) {
    Li = L1
    Ui = U2
    I = (Ui-Li)/(2*(U1-L1)) + (Ui-Li)/(2*(U2-L2))
  }
  if (L2<=L1 & L1<U2 & U1<U2) {
    Li = L1
    Ui = U1
    I = (Ui-Li)/(2*(U1-L1)) + (Ui-Li)/(2*(U2-L2))
  }
  return(I)
}

# Applying the CI overlap to a vector of parameters
vec_int_overlap<-function(vec1,vec2){
  out_vec<-rep(NA,length(vec1))
  for (i in 1:length(out_vec)) {
    out_vec[i]<-interval_overlap(vec1[1],vec2[1],vec1[i],vec2[i])
  }
  return(out_vec)
}

# Wrapper function synthetic data generation 
# (use the function from the synthpop r package if sdgs is not publicly available)
sdgs_synthesis<-function(realdata){
  real.dataset<-realdata %>% na.omit
  yy<-sdgs::synthseq$new()
  # set
  c(yy$verbose, yy$tune , yy$calibrate, yy$parallel) %<-% c(FALSE, FALSE, FALSE, TRUE)
  yy$dataset<-real.dataset
  syn.dataset<-yy$synthesize(nrow(real.dataset))
  yy$finalize()
  return(syn.dataset$syn_data)
}

## Functions to induce marginal bias
Induce_bias<-function(traindata,prop_out){
  cols<-colnames(traindata)
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("Y",paste0("X",1:(p-1)))
  samps<-floor(table(data$X1)[1]*prop_out)
  data = data[-sample(which(data$X1%in%levels(data$X1)[1]),samps),]
  colnames(data)<-cols
  return(data)
}

# Function to generated minority synthetic versions of the biased data (marginal bias)
augment_synth_mar<-function(biased_data,props){
  data_bias2<-biased_data
  data_bias2<-data.frame(apply(data_bias2,2,as.numeric))
  
  # remove missingness in the original dataset
  real.dataset<-data_bias2 %>% na.omit
  
  # Generate the synthetic data 
  synth<-sdgs_synthesis(real.dataset)
  countminor<-length(which(biased_data$Z%in%levels(biased_data$Z)[1]))
  prop_remain<-1-props
  total_size<-floor(countminor/prop_remain)
  synth.sel<-synth[sample(x = which(synth$Z==0),size = (total_size-countminor), replace = T),]
  augment_syn<-rbind(real.dataset,synth.sel)
  #augment_syn[,1]<-as.integer(augment_syn[,1])
  augment_syn$Z<-as.factor(augment_syn$Z)
  augment_syn$X1<-as.factor(augment_syn$X1)
  augment_syn$X2<-as.factor(augment_syn$X2)
  augment_syn$ZX2<-as.factor(augment_syn$ZX2)
  return(augment_syn)
}

##############################################
##############################################
## Methods
# 1. Original (Gold standard)
# 2. Biased data
# 3. Random Undersampling (RUS)
# 4. Random Oversampling (ROS)
# 5. Synthetic Minority Oversampling Technique (SMOTE)
# 6. PS Matching
# 7. RF ensembles
# 8. Synthetic Minority Augmentation SMA

## Simulated Full Cohort
# logit[P(Y=1|)] ~ Z + X1 + X2 + ZX2 + U 

n.sims <- 1000     # Number of simulations 
N <- 5000          # Sample size
prob.Z <- 0.5       # P(Z=1)
prob.X1 <- 0.6      # P(X1=1)
prob.X2.Z0 <- 0.3   # P(X2=1|Z=0)
prob.X2.Z1 <- 0.35  # P(X2=1|Z=1)

beta0 <- log(0.35/(1-0.35))   #  the intercept
betaZ <- log(1.25)            #  0.405  # the exposure of interest
betaX1 <- log(0.3)            #  
betaX2 <- log(2)              #  
betaZX2 <- log(1.25)-log(2)   # -0.111  # (effect modification)
betaU <- log(0.5)             # strong predictor of the outcome

# Declare the list to store the train and test sets
train.sim<-list()
test.sim<-list()

# Simulation
for (i in 1:n.sims){
  Z <- rbinom(n=N, size=1, prob=prob.Z)
  X1 <- rbinom(n=N, size=1, prob=prob.X1)
  # Vector containing the N values of E for sample i
  X2 <- rep(NA, times=N) #modify the exposure variable to something different
  for (j in 1:N){
    if (Z[j]==1){X2[j] <- rbinom(n=1, size=1, prob=prob.X2.Z1)}
    if (Z[j]==0){X2[j] <- rbinom(n=1, size=1, prob=prob.X2.Z0)}
  }
  ZX2 <- Z*X2 # the effect modifier defined 
  ### Generation of Y (dependent variable) ###
  U <- log(abs(rnorm(n = N, mean = 35, sd = 15)))
  
  right <- beta0 + betaZ*Z + betaX1*X1 + betaX2*X2 + betaZX2*ZX2 + betaU*U
  p <- exp(right)/(1 + exp(right))
  
  # Vector of the N generated values of Y for sample i
  Y <- rbinom(n=N, size=1, prob=p)
  pop<-data.frame(Y=Y,Z=as.factor(Z),X1=as.factor(X1),
                  X2=as.factor(X2),ZX2=as.factor(ZX2),U=U)
  
  sel<-sample(1:N,round(N*.3))
  train<-train.sim[[i]]<-pop[-sel,]
  test.sim[[i]]<-pop[sel,]
} 

#### save the train and test sets
saveRDS(train.sim, file = "trainsim.rds")
saveRDS(test.sim, file = "testsim.rds")


##########################################################################
### Bias type I: Marginal bias
mar.fullcohort<-function(props,n.sims){
  bias.mar.auc<-data.frame(Original=rep(NA,n.sims),Biased=rep(NA,n.sims),
                           RUS=rep(NA,n.sims), ROS=rep(NA,n.sims), 
                           SMOTE=rep(NA,n.sims),PSmatch=rep(NA,n.sims),
                           RF=rep(NA,n.sims), SynAug=rep(NA,n.sims))
  bias.mar.SE<-bias.mar.coef<-bias.mar.auc
  for (mar in 1:n.sims) {
    traindata<-train.sim[[mar]]
    testdata<-test.sim[[mar]]
    
    # Unbiased data
    goldstandard<-LRstandard(trainsample = traindata,
                             testsample = testdata)
    bias.mar.auc$Original[mar]<-goldstandard[[1]][1]
    bias.mar.coef$Original[mar]<-goldstandard[[2]][2,1]
    bias.mar.SE$Original[mar]<-goldstandard[[2]][2,2]
    
    # Biased dataset
    data_bias<-Induce_bias(traindata = traindata, prop_out = props)
    augsyn_bias<-augment_synth_mar(biased_data=data_bias, props=props)
    
    LRbias<-LRstandard(trainsample = data_bias, testsample = testdata)
    bias.mar.auc$Biased[mar]<-LRbias[[1]][1]
    bias.mar.coef$Biased[mar]<-LRbias[[2]][2,1]
    bias.mar.SE$Biased[mar]<-LRbias[[2]][2,2]
    
    # RUS model
    data_rus<-RUS_bias(data_bias)
    LRrus<-LRstandard(trainsample = data_rus, testsample = testdata)
    bias.mar.auc$RUS[mar]<-LRrus[[1]][1]
    bias.mar.coef$RUS[mar]<-LRrus[[2]][2,1]
    bias.mar.SE$RUS[mar]<-LRrus[[2]][2,2]
    
    # ROS
    data_ros<-ROS_bias(data_bias)
    LRros<-LRstandard(trainsample = data_ros, testsample = testdata)
    bias.mar.auc$ROS[mar]<-LRros[[1]][1]
    bias.mar.coef$ROS[mar]<-LRros[[2]][2,1]
    bias.mar.SE$ROS[mar]<-LRros[[2]][2,2]
    
    # smote model
    data_smote<-SMOTE_bias(data_bias,perc=100)
    LRsmote<-LRstandard(trainsample = data_smote, testsample = testdata)
    bias.mar.auc$SMOTE[mar]<-LRsmote[[1]][1]
    bias.mar.coef$SMOTE[mar]<-LRsmote[[2]][2,1]
    bias.mar.SE$SMOTE[mar]<-LRsmote[[2]][2,2]
    
    # propensity score model
    LRps<-LR_PSbias(trainsample = data_bias, testsample = testdata)
    bias.mar.auc$PSmatch[mar]<-LRps[[1]][1]
    bias.mar.coef$PSmatch[mar]<-LRps[[2]][2,1]
    bias.mar.SE$PSmatch[mar]<-LRps[[2]][2,2]
    
    # the random forest model
    LRrf<-LR_RFbias(trainsample = data_bias,
                    testsample = testdata)
    bias.mar.auc$RF[mar]<-LRrf[[1]]$value[1]
    
    # SMA model
    Syn_LRbias<-LRstandard(trainsample = augsyn_bias, testsample = testdata)
    bias.mar.auc$SynAug[mar]<-Syn_LRbias[[1]][1]
    bias.mar.coef$SynAug[mar]<-Syn_LRbias[[2]][2,1]
    bias.mar.SE$SynAug[mar]<-Syn_LRbias[[2]][2,2]
  }
  return(list(bias.mar.auc,bias.mar.coef,bias.mar.SE))
}

# Minor function
mar.minor<-function(props,n.sims){
  bias.mar.auc<-data.frame(Original=rep(NA,n.sims),Biased=rep(NA,n.sims),
                           RUS=rep(NA,n.sims), ROS=rep(NA,n.sims), 
                           SMOTE=rep(NA,n.sims),PSmatch=rep(NA,n.sims),
                           RF=rep(NA,n.sims), SynAug=rep(NA,n.sims))
  bias.mar.SE<-bias.mar.coef<-bias.mar.auc
  for (mar in 1:n.sims) {
    traindata<-train.sim[[mar]]
    testdata<-test.sim[[mar]]
    # Unbiased data
    goldstandard<-LRstandard(trainsample = traindata,
                             testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.mar.auc$Original[mar]<-goldstandard[[1]][1]
    bias.mar.coef$Original[mar]<-goldstandard[[2]][2,1]
    bias.mar.SE$Original[mar]<-goldstandard[[2]][2,2]
    
    # Biased dataset
    data_bias<-Induce_bias(traindata = traindata, prop_out = props)
    augsyn_bias<-augment_synth_mar(biased_data=data_bias, props=props)
    
    LRbias<-LRstandard(trainsample = data_bias, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.mar.auc$Biased[mar]<-LRbias[[1]][1]
    bias.mar.coef$Biased[mar]<-LRbias[[2]][2,1]
    bias.mar.SE$Biased[mar]<-LRbias[[2]][2,2]
    
    # RUS model
    data_rus<-RUS_bias(data_bias)
    LRrus<-LRstandard(trainsample = data_rus, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.mar.auc$RUS[mar]<-LRrus[[1]][1]
    bias.mar.coef$RUS[mar]<-LRrus[[2]][2,1]
    bias.mar.SE$RUS[mar]<-LRrus[[2]][2,2]
    
    # ROS
    data_ros<-ROS_bias(data_bias)
    LRros<-LRstandard(trainsample = data_ros, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.mar.auc$ROS[mar]<-LRros[[1]][1]
    bias.mar.coef$ROS[mar]<-LRros[[2]][2,1]
    bias.mar.SE$ROS[mar]<-LRros[[2]][2,2]
    
    # smote model
    data_smote<-SMOTE_bias(data_bias,perc=100)
    LRsmote<-LRstandard(trainsample = data_smote, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.mar.auc$SMOTE[mar]<-LRsmote[[1]][1]
    bias.mar.coef$SMOTE[mar]<-LRsmote[[2]][2,1]
    bias.mar.SE$SMOTE[mar]<-LRsmote[[2]][2,2]
    
    # propensity score model
    LRps<-LR_PSbias2(trainsample = data_bias, testsample = testdata,nclass = 1)
    bias.mar.auc$PSmatch[mar]<-LRps[[1]][1]
    bias.mar.coef$PSmatch[mar]<-LRps[[2]][2,1]
    bias.mar.SE$PSmatch[mar]<-LRps[[2]][2,2]
    
    # the random forest model
    LRrf<-LR_RFbias2(trainsample = data_bias,
                     testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.mar.auc$RF[mar]<-LRrf[[1]]$value[1]
    
    # synthetic data augmentation model
    Syn_LRbias<-LRstandard(trainsample = augsyn_bias, 
                           testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.mar.auc$SynAug[mar]<-Syn_LRbias[[1]][1]
    bias.mar.coef$SynAug[mar]<-Syn_LRbias[[2]][2,1]
    bias.mar.SE$SynAug[mar]<-Syn_LRbias[[2]][2,2]
  }
  return(list(bias.mar.auc,bias.mar.coef,bias.mar.SE))
}


# Major function
mar.major<-function(props,n.sims){
  bias.mar.auc<-data.frame(Original=rep(NA,n.sims),Biased=rep(NA,n.sims),
                           RUS=rep(NA,n.sims), ROS=rep(NA,n.sims), 
                           SMOTE=rep(NA,n.sims),PSmatch=rep(NA,n.sims),
                           RF=rep(NA,n.sims), SynAug=rep(NA,n.sims))
  bias.mar.SE<-bias.mar.coef<-bias.mar.auc
  for (mar in 1:n.sims) {
    traindata<-train.sim[[mar]]
    testdata<-test.sim[[mar]]
    # Unbiased data
    goldstandard<-LRstandard(trainsample = traindata,
                             testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.mar.auc$Original[mar]<-goldstandard[[1]][1]
    bias.mar.coef$Original[mar]<-goldstandard[[2]][2,1]
    bias.mar.SE$Original[mar]<-goldstandard[[2]][2,2]
    
    # Biased dataset
    data_bias<-Induce_bias(traindata = traindata, prop_out = props)
    augsyn_bias<-augment_synth_mar(biased_data=data_bias, props=props)
    
    LRbias<-LRstandard(trainsample = data_bias, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.mar.auc$Biased[mar]<-LRbias[[1]][1]
    bias.mar.coef$Biased[mar]<-LRbias[[2]][2,1]
    bias.mar.SE$Biased[mar]<-LRbias[[2]][2,2]
    
    # RUS model
    data_rus<-RUS_bias(data_bias)
    LRrus<-LRstandard(trainsample = data_rus, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.mar.auc$RUS[mar]<-LRrus[[1]][1]
    bias.mar.coef$RUS[mar]<-LRrus[[2]][2,1]
    bias.mar.SE$RUS[mar]<-LRrus[[2]][2,2]
    
    # ROS
    data_ros<-ROS_bias(data_bias)
    LRros<-LRstandard(trainsample = data_ros, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.mar.auc$ROS[mar]<-LRros[[1]][1]
    bias.mar.coef$ROS[mar]<-LRros[[2]][2,1]
    bias.mar.SE$ROS[mar]<-LRros[[2]][2,2]
    
    # smote model
    data_smote<-SMOTE_bias(data_bias,perc=100)
    LRsmote<-LRstandard(trainsample = data_smote, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.mar.auc$SMOTE[mar]<-LRsmote[[1]][1]
    bias.mar.coef$SMOTE[mar]<-LRsmote[[2]][2,1]
    bias.mar.SE$SMOTE[mar]<-LRsmote[[2]][2,2]
    
    # propensity score model
    LRps<-LR_PSbias2(trainsample = data_bias, testsample = testdata,nclass = 2)
    bias.mar.auc$PSmatch[mar]<-LRps[[1]][1]
    bias.mar.coef$PSmatch[mar]<-LRps[[2]][2,1]
    bias.mar.SE$PSmatch[mar]<-LRps[[2]][2,2]
    
    # the random forest model
    LRrf<-LR_RFbias2(trainsample = data_bias,
                     testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.mar.auc$RF[mar]<-LRrf[[1]]$value[1]
    
    # synthetic data augmentation model
    Syn_LRbias<-LRstandard(trainsample = augsyn_bias, 
                           testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.mar.auc$SynAug[mar]<-Syn_LRbias[[1]][1]
    bias.mar.coef$SynAug[mar]<-Syn_LRbias[[2]][2,1]
    bias.mar.SE$SynAug[mar]<-Syn_LRbias[[2]][2,2]
  }
  return(list(bias.mar.auc,bias.mar.coef,bias.mar.SE))
}


nsims <-1000 # Number of simulated samples to evaluate

## Full cohort
mar.full.15 = mar.fullcohort(props = 0.15,n.sims = nsims)
mar.full.30 = mar.fullcohort(props = 0.3,n.sims = nsims)
mar.full.50 = mar.fullcohort(props = 0.5,n.sims = nsims)
mar.full.80 = mar.fullcohort(props = 0.8,n.sims = nsims)
mar.full.95 = mar.fullcohort(props = 0.95,n.sims = nsims)


saveRDS(mar.full.15, file = "marfull15.rds")
saveRDS(mar.full.30, file = "marfull30.rds")
saveRDS(mar.full.50, file = "marfull50.rds")
saveRDS(mar.full.80, file = "marfull80.rds")
saveRDS(mar.full.95, file = "marfull95.rds")

# Minor
mar.minor.30 = mar.minor(props = 0.3,n.sims = nsims)
mar.minor.50 = mar.minor(props = 0.5,n.sims = nsims)
mar.minor.80 = mar.minor(props = 0.8,n.sims = nsims)

saveRDS(mar.minor.30, file = "marminor30.rds")
saveRDS(mar.minor.50, file = "marminor50.rds")
saveRDS(mar.minor.80, file = "marminor80.rds")

# Major
mar.major.30 = mar.major(props = 0.3,n.sims = nsims)
mar.major.50 = mar.major(props = 0.5,n.sims = nsims)
mar.major.80 = mar.major(props = 0.8,n.sims = nsims)

saveRDS(mar.major.30, file = "marmajor30.rds")
saveRDS(mar.major.50, file = "marmajor50.rds")
saveRDS(mar.major.80, file = "marmajor80.rds")


########################################################################
### Bias type II:  Conditional bias I
Induce_bias_1<-function(traindata,prop_out){
  cols<-colnames(traindata)
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("Y",paste0("X",1:(p-1)))
  selval<-which(data$X1%in%levels(data$X1)[1]&
                  data$X2%in%levels(data$X2)[1])
  samps<-floor(length(selval)[1]*prop_out)
  data = data[sample(-selval,samps),]
  colnames(data)<-cols
  return(data)
}

# Define the function for generating the synthetic sample
augment_synth_cond1<-function(biased_data,props){
  data_bias2<-biased_data
  data_bias2<-data.frame(apply(data_bias2,2,as.numeric))
  # remove missing
  real.dataset<-data_bias2 %>% na.omit
  
  # Generate the synthetic data with CART
  synth<-sdgs_synthesis(real.dataset)
  countminor<-length(which(data_bias2$Z==0&data_bias2$X1==0))
  prop_remain<-1-props
  total_size<-floor(countminor/prop_remain)
  
  synth.sel<-synth[sample(x = which(synth$Z==0&synth$X1==0),
                          size = (total_size-countminor), replace = T),]
  augment_syn<-rbind(real.dataset,synth.sel)
  augment_syn$Z<-as.factor(augment_syn$Z)
  augment_syn$X1<-as.factor(augment_syn$X1)
  augment_syn$X2<-as.factor(augment_syn$X2)
  augment_syn$ZX2<-as.factor(augment_syn$ZX2)
  return(augment_syn)
}

cond1.fullcohort<-function(props,n.sims){
  bias.auc<-data.frame(Original=rep(NA,n.sims),Biased=rep(NA,n.sims),
                       RUS=rep(NA,n.sims), ROS=rep(NA,n.sims), 
                       SMOTE=rep(NA,n.sims),PSmatch=rep(NA,n.sims),
                       RF=rep(NA,n.sims), SynAug=rep(NA,n.sims))
  bias.SE<-bias.coef<-bias.auc
  for (cc in 1:n.sims) {
    traindata<-train.sim[[cc]]
    testdata<-test.sim[[cc]]
    
    # Unbiased data
    goldstandard<-LRstandard(trainsample = traindata,
                             testsample = testdata)
    bias.auc$Original[cc]<-goldstandard[[1]][1]
    bias.coef$Original[cc]<-goldstandard[[2]][2,1]
    bias.SE$Original[cc]<-goldstandard[[2]][2,2]
    
    # Biased dataset
    data_bias<-Induce_bias_1(traindata = traindata, prop_out = props)
    augsyn_bias<-augment_synth_cond1(biased_data=data_bias, props=props)
    
    
    LRbias<-LRstandard(trainsample = data_bias, testsample = testdata)
    bias.auc$Biased[cc]<-LRbias[[1]][1]
    bias.coef$Biased[cc]<-LRbias[[2]][2,1]
    bias.SE$Biased[cc]<-LRbias[[2]][2,2]
    
    # RUS model
    data_rus<-RUS_bias(data_bias)
    LRrus<-LRstandard(trainsample = data_rus, testsample = testdata)
    bias.auc$RUS[cc]<-LRrus[[1]][1]
    bias.coef$RUS[cc]<-LRrus[[2]][2,1]
    bias.SE$RUS[cc]<-LRrus[[2]][2,2]
    
    # ROS
    data_ros<-ROS_bias(data_bias)
    LRros<-LRstandard(trainsample = data_ros, testsample = testdata)
    bias.auc$ROS[cc]<-LRros[[1]][1]
    bias.coef$ROS[cc]<-LRros[[2]][2,1]
    bias.SE$ROS[cc]<-LRros[[2]][2,2]
    
    # smote model
    data_smote<-SMOTE_bias(data_bias,perc=100)
    LRsmote<-LRstandard(trainsample = data_smote, testsample = testdata)
    bias.auc$SMOTE[cc]<-LRsmote[[1]][1]
    bias.coef$SMOTE[cc]<-LRsmote[[2]][2,1]
    bias.SE$SMOTE[cc]<-LRsmote[[2]][2,2]
    
    # propensity score model
    LRps<-LR_PSbias(trainsample = data_bias, testsample = testdata)
    bias.auc$PSmatch[cc]<-LRps[[1]][1]
    bias.coef$PSmatch[cc]<-LRps[[2]][2,1]
    bias.SE$PSmatch[cc]<-LRps[[2]][2,2]
    
    # the random forest model
    LRrf<-LR_RFbias(trainsample = data_bias,
                    testsample = testdata)
    bias.auc$RF[cc]<-LRrf[[1]]$value[1]
    
    
    # synthetic data augmentation model
    Syn_LRbias<-LRstandard(trainsample = augsyn_bias, testsample = testdata)
    bias.auc$SynAug[cc]<-Syn_LRbias[[1]][1]
    bias.coef$SynAug[cc]<-Syn_LRbias[[2]][2,1]
    bias.SE$SynAug[cc]<-Syn_LRbias[[2]][2,2]
  }
  return(list(bias.auc,bias.coef,bias.SE))
}

# Minority group 
cond1.minor<-function(props,n.sims){
  bias.auc<-data.frame(Original=rep(NA,n.sims),Biased=rep(NA,n.sims),
                       RUS=rep(NA,n.sims), ROS=rep(NA,n.sims), 
                       SMOTE=rep(NA,n.sims),PSmatch=rep(NA,n.sims),
                       RF=rep(NA,n.sims), SynAug=rep(NA,n.sims))
  bias.SE<-bias.coef<-bias.auc
  for (cc in 1:n.sims) {
    traindata<-train.sim[[cc]]
    testdata<-test.sim[[cc]]
    # Unbiased data
    goldstandard<-LRstandard(trainsample = traindata,
                             testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$Original[cc]<-goldstandard[[1]][1]
    bias.coef$Original[cc]<-goldstandard[[2]][2,1]
    bias.SE$Original[cc]<-goldstandard[[2]][2,2]
    
    # Biased dataset
    data_bias<-Induce_bias_1(traindata = traindata, prop_out = props)
    augsyn_bias<-augment_synth_cond1(biased_data=data_bias, props=props)
    
    LRbias<-LRstandard(trainsample = data_bias, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$Biased[cc]<-LRbias[[1]][1]
    bias.coef$Biased[cc]<-LRbias[[2]][2,1]
    bias.SE$Biased[cc]<-LRbias[[2]][2,2]
    
    # RUS model
    data_rus<-RUS_bias(data_bias)
    LRrus<-LRstandard(trainsample = data_rus, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$RUS[cc]<-LRrus[[1]][1]
    bias.coef$RUS[cc]<-LRrus[[2]][2,1]
    bias.SE$RUS[cc]<-LRrus[[2]][2,2]
    
    # ROS
    data_ros<-ROS_bias(data_bias)
    LRros<-LRstandard(trainsample = data_ros, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$ROS[cc]<-LRros[[1]][1]
    bias.coef$ROS[cc]<-LRros[[2]][2,1]
    bias.SE$ROS[cc]<-LRros[[2]][2,2]
    
    # smote model
    data_smote<-SMOTE_bias(data_bias,perc=100)
    LRsmote<-LRstandard(trainsample = data_smote, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$SMOTE[cc]<-LRsmote[[1]][1]
    bias.coef$SMOTE[cc]<-LRsmote[[2]][2,1]
    bias.SE$SMOTE[cc]<-LRsmote[[2]][2,2]
    
    # propensity score model
    LRps<-LR_PSbias2(trainsample = data_bias, testsample = testdata,nclass = 1)
    bias.auc$PSmatch[cc]<-LRps[[1]][1]
    bias.coef$PSmatch[cc]<-LRps[[2]][2,1]
    bias.SE$PSmatch[cc]<-LRps[[2]][2,2]
    
    # the random forest model
    LRrf<-LR_RFbias2(trainsample = data_bias,
                     testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$RF[cc]<-LRrf[[1]]$value[1]
    
    # synthetic data augmentation model
    Syn_LRbias<-LRstandard(trainsample = augsyn_bias, testsample = testdata)
    bias.auc$SynAug[cc]<-Syn_LRbias[[1]][1]
    bias.coef$SynAug[cc]<-Syn_LRbias[[2]][2,1]
    bias.SE$SynAug[cc]<-Syn_LRbias[[2]][2,2]
    
  }
  return(list(bias.auc,bias.coef,bias.SE))
}

# Major group 
cond1.major<-function(props,n.sims){
  bias.auc<-data.frame(Original=rep(NA,n.sims),Biased=rep(NA,n.sims),
                       RUS=rep(NA,n.sims), ROS=rep(NA,n.sims), 
                       SMOTE=rep(NA,n.sims),PSmatch=rep(NA,n.sims),
                       RF=rep(NA,n.sims),SynAug=rep(NA,n.sims))
  bias.SE<-bias.coef<-bias.auc
  for (cc in 1:n.sims) {
    traindata<-train.sim[[cc]]
    testdata<-test.sim[[cc]]
    # Unbiased data
    goldstandard<-LRstandard(trainsample = traindata,
                             testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$Original[cc]<-goldstandard[[1]][1]
    bias.coef$Original[cc]<-goldstandard[[2]][2,1]
    bias.SE$Original[cc]<-goldstandard[[2]][2,2]
    
    # Biased dataset
    data_bias<-Induce_bias_1(traindata = traindata, prop_out = props)
    augsyn_bias<-augment_synth_cond1(biased_data=data_bias, props=props)
    
    LRbias<-LRstandard(trainsample = data_bias, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$Biased[cc]<-LRbias[[1]][1]
    bias.coef$Biased[cc]<-LRbias[[2]][2,1]
    bias.SE$Biased[cc]<-LRbias[[2]][2,2]
    
    # RUS model
    data_rus<-RUS_bias(data_bias)
    LRrus<-LRstandard(trainsample = data_rus, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$RUS[cc]<-LRrus[[1]][1]
    bias.coef$RUS[cc]<-LRrus[[2]][2,1]
    bias.SE$RUS[cc]<-LRrus[[2]][2,2]
    
    # ROS
    data_ros<-ROS_bias(data_bias)
    LRros<-LRstandard(trainsample = data_ros, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$ROS[cc]<-LRros[[1]][1]
    bias.coef$ROS[cc]<-LRros[[2]][2,1]
    bias.SE$ROS[cc]<-LRros[[2]][2,2]
    
    # smote model
    data_smote<-SMOTE_bias(data_bias,perc=100)
    LRsmote<-LRstandard(trainsample = data_smote, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$SMOTE[cc]<-LRsmote[[1]][1]
    bias.coef$SMOTE[cc]<-LRsmote[[2]][2,1]
    bias.SE$SMOTE[cc]<-LRsmote[[2]][2,2]
    
    # propensity score model
    LRps<-LR_PSbias2(trainsample = data_bias, testsample = testdata,nclass = 2)
    bias.auc$PSmatch[cc]<-LRps[[1]][1]
    bias.coef$PSmatch[cc]<-LRps[[2]][2,1]
    bias.SE$PSmatch[cc]<-LRps[[2]][2,2]
    
    # the random forest model
    LRrf<-LR_RFbias2(trainsample = data_bias,
                     testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$RF[cc]<-LRrf[[1]]$value[1]
    
    # synthetic data augmentation model
    Syn_LRbias<-LRstandard(trainsample = augsyn_bias, testsample = testdata)
    bias.auc$SynAug[cc]<-Syn_LRbias[[1]][1]
    bias.coef$SynAug[cc]<-Syn_LRbias[[2]][2,1]
    bias.SE$SynAug[cc]<-Syn_LRbias[[2]][2,2]
    
  }
  return(list(bias.auc,bias.coef,bias.SE))
}


# Full Cohort 
cond1.full.15 = cond1.fullcohort(props = 0.15,n.sims = nsims)
cond1.full.30 = cond1.fullcohort(props = 0.3,n.sims = nsims)
cond1.full.50 = cond1.fullcohort(props = 0.5,n.sims = nsims)
cond1.full.80 = cond1.fullcohort(props = 0.8,n.sims = nsims)
cond1.full.95 = cond1.fullcohort(props = 0.95,n.sims = nsims)

saveRDS(cond1.full.15, file = "cond1full15.rds")
saveRDS(cond1.full.30, file = "cond1full30.rds")
saveRDS(cond1.full.50, file = "cond1full50.rds")
saveRDS(cond1.full.80, file = "cond1full80.rds")
saveRDS(cond1.full.95, file = "cond1full95.rds")

# Minority class AUC
cond1.minor.30 = cond1.minor(props = 0.3,n.sims = nsims)
cond1.minor.50 = cond1.minor(props = 0.5,n.sims = nsims)
cond1.minor.80 = cond1.minor(props = 0.8,n.sims = nsims)

saveRDS(cond1.minor.30, file = "cond1minor30.rds")
saveRDS(cond1.minor.50, file = "cond1minor50.rds")
saveRDS(cond1.minor.80, file = "cond1minor80.rds")

# Major class AUC
cond1.major.30 = cond1.major(props = 0.3,n.sims = nsims)
cond1.major.50 = cond1.major(props = 0.5,n.sims = nsims)
cond1.major.80 = cond1.major(props = 0.8,n.sims = nsims)

saveRDS(cond1.major.30, file = "cond1major30.rds")
saveRDS(cond1.major.50, file = "cond1major50.rds")
saveRDS(cond1.major.80, file = "cond1major80.rds")


##############################################################################
### Bias type III: Conditional bias 2
Induce_bias_2<-function(traindata,prop_out){
  cols<-colnames(traindata)
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("Y",paste0("X",1:(p-1)))
  selval<-which(data$X1%in%levels(data$X1)[1]&
                  data$X3%in%levels(data$X3)[1])
  samps<-floor(length(selval)[1]*prop_out)
  data = data[sample(-selval,samps),]
  colnames(data)<-cols
  return(data)
}

# Define the function for generating the synthetic sampole
augment_synth_cond2<-function(biased_data,props){
  data_bias2<-biased_data
  data_bias2<-data.frame(apply(data_bias2,2,as.numeric))
  # remove missing
  real.dataset<-data_bias2 %>% na.omit
  # Generate the synthetic data with CART
  synth<-sdgs_synthesis(real.dataset)
  countminor<-length(which(data_bias2$Z==0&data_bias2$X2==0))
  prop_remain<-1-props
  total_size<-floor(countminor/prop_remain)
  synth.sel<-synth[sample(x = which(synth$Z==0&synth$X2==0),
                          size = (total_size-countminor), replace = T),]
  augment_syn<-rbind(real.dataset,synth.sel)
  augment_syn$Z<-as.factor(augment_syn$Z)
  augment_syn$X1<-as.factor(augment_syn$X1)
  augment_syn$X2<-as.factor(augment_syn$X2)
  augment_syn$ZX2<-as.factor(augment_syn$ZX2)
  return(augment_syn)
}

### select the proportions
cond2.fullcohort<-function(props,n.sims){
  bias.auc<-data.frame(Original=rep(NA,n.sims),Biased=rep(NA,n.sims),
                       RUS=rep(NA,n.sims), ROS=rep(NA,n.sims), 
                       SMOTE=rep(NA,n.sims),PSmatch=rep(NA,n.sims),
                       RF=rep(NA,n.sims), SynAug=rep(NA,n.sims))
  bias.SE<-bias.coef<-bias.auc
  for (cc in 1:n.sims) {
    traindata<-train.sim[[cc]]
    testdata<-test.sim[[cc]]
    
    # Unbiased data
    goldstandard<-LRstandard(trainsample = traindata,
                             testsample = testdata)
    bias.auc$Original[cc]<-goldstandard[[1]][1]
    bias.coef$Original[cc]<-goldstandard[[2]][2,1]
    bias.SE$Original[cc]<-goldstandard[[2]][2,2]
    
    # Biased dataset
    data_bias<-Induce_bias_2(traindata = traindata, prop_out = props)
    augsyn_bias<-augment_synth_cond2(biased_data=data_bias, props=props)
    
    LRbias<-LRstandard(trainsample = data_bias, testsample = testdata)
    bias.auc$Biased[cc]<-LRbias[[1]][1]
    bias.coef$Biased[cc]<-LRbias[[2]][2,1]
    bias.SE$Biased[cc]<-LRbias[[2]][2,2]
    
    # RUS model
    data_rus<-RUS_bias(data_bias)
    LRrus<-LRstandard(trainsample = data_rus, testsample = testdata)
    bias.auc$RUS[cc]<-LRrus[[1]][1]
    bias.coef$RUS[cc]<-LRrus[[2]][2,1]
    bias.SE$RUS[cc]<-LRrus[[2]][2,2]
    
    # ROS
    data_ros<-ROS_bias(data_bias)
    LRros<-LRstandard(trainsample = data_ros, testsample = testdata)
    bias.auc$ROS[cc]<-LRros[[1]][1]
    bias.coef$ROS[cc]<-LRros[[2]][2,1]
    bias.SE$ROS[cc]<-LRros[[2]][2,2]
    
    # smote model
    data_smote<-SMOTE_bias(data_bias,perc=100)
    LRsmote<-LRstandard(trainsample = data_smote, testsample = testdata)
    bias.auc$SMOTE[cc]<-LRsmote[[1]][1]
    bias.coef$SMOTE[cc]<-LRsmote[[2]][2,1]
    bias.SE$SMOTE[cc]<-LRsmote[[2]][2,2]
    
    # propensity score model
    LRps<-LR_PSbias(trainsample = data_bias, testsample = testdata)
    bias.auc$PSmatch[cc]<-LRps[[1]][1]
    bias.coef$PSmatch[cc]<-LRps[[2]][2,1]
    bias.SE$PSmatch[cc]<-LRps[[2]][2,2]
    
    # the random forest model
    LRrf<-LR_RFbias(trainsample = data_bias,
                    testsample = testdata)
    bias.auc$RF[cc]<-LRrf[[1]]$value[1]
    
    # synthetic data augmentation model
    Syn_LRbias<-LRstandard(trainsample = augsyn_bias, testsample = testdata)
    bias.auc$SynAug[cc]<-Syn_LRbias[[1]][1]
    bias.coef$SynAug[cc]<-Syn_LRbias[[2]][2,1]
    bias.SE$SynAug[cc]<-Syn_LRbias[[2]][2,2]
    
  }
  return(list(bias.auc,bias.coef,bias.SE))
}

# Minor function
cond2.minor<-function(props,n.sims){
  bias.auc<-data.frame(Original=rep(NA,n.sims),Biased=rep(NA,n.sims),
                       RUS=rep(NA,n.sims), ROS=rep(NA,n.sims), 
                       SMOTE=rep(NA,n.sims),PSmatch=rep(NA,n.sims),
                       RF=rep(NA,n.sims), SynAug=rep(NA,n.sims))
  bias.SE<-bias.coef<-bias.auc
  for (cc in 1:n.sims) {
    traindata<-train.sim[[cc]]
    testdata<-test.sim[[cc]]
    # Unbiased data
    goldstandard<-LRstandard(trainsample = traindata,
                             testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$Original[cc]<-goldstandard[[1]][1]
    bias.coef$Original[cc]<-goldstandard[[2]][2,1]
    bias.SE$Original[cc]<-goldstandard[[2]][2,2]
    
    # Biased dataset
    data_bias<-Induce_bias_2(traindata = traindata, prop_out = props)
    augsyn_bias<-augment_synth_cond2(biased_data=data_bias, props=props)
    
    LRbias<-LRstandard(trainsample = data_bias, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$Biased[cc]<-LRbias[[1]][1]
    bias.coef$Biased[cc]<-LRbias[[2]][2,1]
    bias.SE$Biased[cc]<-LRbias[[2]][2,2]
    
    # RUS model
    data_rus<-RUS_bias(data_bias)
    LRrus<-LRstandard(trainsample = data_rus, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$RUS[cc]<-LRrus[[1]][1]
    bias.coef$RUS[cc]<-LRrus[[2]][2,1]
    bias.SE$RUS[cc]<-LRrus[[2]][2,2]
    
    # ROS
    data_ros<-ROS_bias(data_bias)
    LRros<-LRstandard(trainsample = data_ros, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$ROS[cc]<-LRros[[1]][1]
    bias.coef$ROS[cc]<-LRros[[2]][2,1]
    bias.SE$ROS[cc]<-LRros[[2]][2,2]
    
    # smote model
    data_smote<-SMOTE_bias(data_bias,perc=100)
    LRsmote<-LRstandard(trainsample = data_smote, testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$SMOTE[cc]<-LRsmote[[1]][1]
    bias.coef$SMOTE[cc]<-LRsmote[[2]][2,1]
    bias.SE$SMOTE[cc]<-LRsmote[[2]][2,2]
    
    # propensity score model
    LRps<-LR_PSbias2(trainsample = data_bias, testsample = testdata,nclass = 1)
    bias.auc$PSmatch[cc]<-LRps[[1]][1]
    bias.coef$PSmatch[cc]<-LRps[[2]][2,1]
    bias.SE$PSmatch[cc]<-LRps[[2]][2,2]
    
    # the random forest model
    LRrf<-LR_RFbias2(trainsample = data_bias,
                     testsample = testdata[testdata$Z==levels(testdata$Z)[1],])
    bias.auc$RF[cc]<-LRrf[[1]]$value[1]
    
    #synthetic data augmentation model
    Syn_LRbias<-LRstandard(trainsample = augsyn_bias, testsample = testdata)
    bias.auc$SynAug[cc]<-Syn_LRbias[[1]][1]
    bias.coef$SynAug[cc]<-Syn_LRbias[[2]][2,1]
    bias.SE$SynAug[cc]<-Syn_LRbias[[2]][2,2]
  }
  return(list(bias.auc,bias.coef,bias.SE))
}

# Major class funciton
cond2.major<-function(props,n.sims){
  bias.auc<-data.frame(Original=rep(NA,n.sims),Biased=rep(NA,n.sims),
                       RUS=rep(NA,n.sims), ROS=rep(NA,n.sims), 
                       SMOTE=rep(NA,n.sims),PSmatch=rep(NA,n.sims),
                       RF=rep(NA,n.sims), SynAug=rep(NA,n.sims))
  bias.SE<-bias.coef<-bias.auc
  for (cc in 1:n.sims) {
    traindata<-train.sim[[cc]]
    testdata<-test.sim[[cc]]
    # Unbiased data
    goldstandard<-LRstandard(trainsample = traindata,
                             testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$Original[cc]<-goldstandard[[1]][1]
    bias.coef$Original[cc]<-goldstandard[[2]][2,1]
    bias.SE$Original[cc]<-goldstandard[[2]][2,2]
    
    # Biased dataset
    data_bias<-Induce_bias_2(traindata = traindata, prop_out = props)
    augsyn_bias<-augment_synth_cond2(biased_data=data_bias, props=props)
    
    LRbias<-LRstandard(trainsample = data_bias, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$Biased[cc]<-LRbias[[1]][1]
    bias.coef$Biased[cc]<-LRbias[[2]][2,1]
    bias.SE$Biased[cc]<-LRbias[[2]][2,2]
    
    # RUS model
    data_rus<-RUS_bias(data_bias)
    LRrus<-LRstandard(trainsample = data_rus, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$RUS[cc]<-LRrus[[1]][1]
    bias.coef$RUS[cc]<-LRrus[[2]][2,1]
    bias.SE$RUS[cc]<-LRrus[[2]][2,2]
    
    # ROS
    data_ros<-ROS_bias(data_bias)
    LRros<-LRstandard(trainsample = data_ros, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$ROS[cc]<-LRros[[1]][1]
    bias.coef$ROS[cc]<-LRros[[2]][2,1]
    bias.SE$ROS[cc]<-LRros[[2]][2,2]
    
    # smote model
    data_smote<-SMOTE_bias(data_bias,perc=100)
    LRsmote<-LRstandard(trainsample = data_smote, testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$SMOTE[cc]<-LRsmote[[1]][1]
    bias.coef$SMOTE[cc]<-LRsmote[[2]][2,1]
    bias.SE$SMOTE[cc]<-LRsmote[[2]][2,2]
    
    # propensity score model
    LRps<-LR_PSbias2(trainsample = data_bias, testsample = testdata,nclass = 2)
    bias.auc$PSmatch[cc]<-LRps[[1]][1]
    bias.coef$PSmatch[cc]<-LRps[[2]][2,1]
    bias.SE$PSmatch[cc]<-LRps[[2]][2,2]
    
    # the random forest model
    LRrf<-LR_RFbias2(trainsample = data_bias,
                     testsample = testdata[testdata$Z==levels(testdata$Z)[2],])
    bias.auc$RF[cc]<-LRrf[[1]]$value[1]
    
    # synthetic data augmentation model
    Syn_LRbias<-LRstandard(trainsample = augsyn_bias, testsample = testdata)
    bias.auc$SynAug[cc]<-Syn_LRbias[[1]][1]
    bias.coef$SynAug[cc]<-Syn_LRbias[[2]][2,1]
    bias.SE$SynAug[cc]<-Syn_LRbias[[2]][2,2]
  }
  return(list(bias.auc,bias.coef,bias.SE))
}


# Full Cohort 
cond2.full.15 = cond2.fullcohort(props = 0.15,n.sims = nsims)
cond2.full.30 = cond2.fullcohort(props = 0.3,n.sims = nsims)
cond2.full.50 = cond2.fullcohort(props = 0.5,n.sims = nsims)
cond2.full.80 = cond2.fullcohort(props = 0.8,n.sims = nsims)
cond2.full.95 = cond2.fullcohort(props = 0.95,n.sims = nsims)

saveRDS(cond2.full.15, file = "cond2full15.rds")
saveRDS(cond2.full.30, file = "cond2full30.rds")
saveRDS(cond2.full.50, file = "cond2full50.rds")
saveRDS(cond2.full.80, file = "cond2full80.rds")
saveRDS(cond2.full.95, file = "cond2full95.rds")

# Minority class AUC
cond2.minor.30 = cond2.minor(props = 0.3,n.sims = nsims)
cond2.minor.50 = cond2.minor(props = 0.5,n.sims = nsims)
cond2.minor.80 = cond2.minor(props = 0.8,n.sims = nsims)

saveRDS(cond2.minor.30, file = "cond2minor30.rds")
saveRDS(cond2.minor.50, file = "cond2minor50.rds")
saveRDS(cond2.minor.80, file = "cond2minor80.rds")

# Major class AUC
cond2.major.30 = cond2.major(props = 0.3,n.sims = nsims)
cond2.major.50 = cond2.major(props = 0.5,n.sims = nsims)
cond2.major.80 = cond2.major(props = 0.8,n.sims = nsims)

saveRDS(cond2.major.30, file = "cond2major30.rds")
saveRDS(cond2.major.50, file = "cond2major50.rds")
saveRDS(cond2.major.80, file = "cond2major80.rds")


### For missing proportions of 15% and 95%, 
# specify the corresponding missing proportions in the functions
